#!/usr/bin/perl -w

#### LAST UPDATED: Feb, 2004 ####
# choose.pl version 0.1

###########################################################################
#
# This program calls the appropriate parse feature learning programs as
# specified by PARSEFEATURES which are hyphen separated digits which
# map to the following parse features:
#
#	1	head word of phrase housing the target word
#	2	phrase of target word
#	3	head of parent phrase
#	4	parent phrase of target word
#	0	no parse feature
#
# Thus, if we wanted to use just the head and parent words, PARSEFEATURES
# would be 1-3
#
# USAGE :  choose.pl [OPTIONS] PARSEFEATURES 
#
#	--word		target word. Mandatory.
#	--mincount	any parse feature must must exist more than this
#			minimum number in training data. Defaults to 0.
#       --version       Prints the version number.
#       --help          Prints help message.
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("word=s", "mincount=i", "version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version ) {
    $opt_version = 1;
    &showVersion();
    exit; }

# if target word has not been specified, exit
if (defined $opt_word) {
    $wrd = $opt_word;
} else {
    print "Word not specified\n";
    &usage();
    exit; }

# if mincount has not been specified, exit
if (defined $opt_mincount) {
    $MIN = $opt_mincount;
} else {
    print "mincount not specified. Defaulting to 0.\n";
    $MIN = 0; }

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 0 ) {
    print "Insufficient command line arguments.\n";
    &usage();
    exit;}

# Check if mincount is a positive integer
if($MIN !~ /^\d+$/) 	{
    print "MIN must be a positive integer.\n";
    &usage();
    exit;}

# Accept PARSEFEATURES 
$parftrs = shift();

@indfeatures = split(/-/, $parftrs);
$ctr = 0;
foreach $feature (@indfeatures) {
	if($feature !~ /^\d+$/) {
		print "Parse Feature $feature is not numeric. Ignoring it.\n";
		$ctr++;
		next;
	}
	system("learnParse$feature.pl --mincount $MIN $wrd-training.learn.$feature-opt $wrd-training.count");

	# $ctr++;
}

# Subroutine to print help message
sub help{
    print "Usage: choose.pl [OPTIONS] PARSEFEATURES\n\n";
    print "This program calls the appropriate parse feature learning programs as
 specified by PARSEFEATURES.\n\n";
    print "OPTIONS:\n\n";
    print "  --word       target word. Mandatory.\n";
    print "  --mincount   any parse feature must must exist more than this\n";
    print "               minimum number in SOURCE. Defaults to 0.\n";
    print "  --version    Prints the version number\n";
    print "  --help       Prints the help message\n\n";}

sub usage{
    print "Usage: choose.pl [OPTIONS] PARSEFEATURES\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type choose.pl --help for help.\n";}
# function to output the version number
sub showVersion
{
    print STDERR "choose.pl      -        version 0.1\n";
    print STDERR "A component of SyntaLex 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad and Ted Pedersen\n";
}

