#!/usr/bin/perl -w

#### LAST UPDATED: Aug, 2003 ####
# learnParse4.pl version 0.1

###########################################################################
#
# This program learns parent phrase features from a given training file, 
# SOURCE. Those features which occur at least a user
# specified number of times (MIN) constitute a learned feature. Each
# learned feature is printed to the OUTPUT file on a new line. The feature
# is preceeded with 'PHDpos'.
#
# USAGE :  learnParse4.pl [OPTIONS] OUTPUT SOURCE
#
# OPTIONS:
#     --mincount MIN     the feature must occur atleast MIN times to
#                        be selected. Default 0.
#     --help             Prints the help message.\n";
#     --version          Prints the version number.\n\n";
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("mincount=i", "version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}
# Capture the mincount
if ( defined $opt_mincount )
{
    $MIN = $opt_mincount;
    # CHECK for valid MIN
}
else {
    print "Assuming mincount to be 0\n";
    $MIN = 0;
}

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 1 ) {
    print "Insufficient command line arguments\n";
    &usage();
    exit;}

$opfl = shift();
$inpfl = shift();

if(-f $opfl) {
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
        print "Overwriting the file.\n"; }
    else{
        exit; } }

if(!(-f $inpfl)) {
    print "SOURCE file does not exist.\n";
        exit; }

# Open the various files
open(INP,"<$inpfl");
open(OP,">$opfl");

# Read lines from input file
while($ln = <INP>) {
	# Ignore blank lines
	if($ln =~ /^\s*$/){
	    next;}

	# Capture pos at the position
	$hdwd = "UNDEFINED";
	# Present phrase head word = Parse tag + Multiple <P..>..</P> tags + possible non <P= or </P tag + <head
	 # if($ln =~ /\s<P=\"[^~]+~([^~]+)~[^~]+~[^\"]+\">\s((([^<]|<[^P]|<P[^=])[^\s]*\s)*<P=\"[^~]+~[^~]+~[^~]+~[^\"]+\">\s(([^<]|<[^P]|<P[^=])[^\s]*\s)*<\/P>\s(([^<]|<[^P]|<P[^=])[^\s]*\s)*)*(([^<]|<[^P\/]|<P[^=]|<\/[^P])[^\s]*\s)*<head[^>]*>\s*([^\s<]+)/) {

	# Parent phrase head word = Parse tag + Multiple <P..>..</P> tags + Present phrase
	# if($ln =~ /\s<P=\"([^~]+)~[^~]+~[^~]+~[^\"]+\">\s((([^<]|<[^P]|<P[^=])[^\s]*\s)*<P=\"[^~]+~[^~]+~[^~]+~[^\"]+\">\s(([^<]|<[^P]|<P[^=])[^\s]*\s)*<\/P>\s(([^<]|<[^P]|<P[^=])[^\s]*\s)*)*(([^<]|<[^P\/]|<P[^=]|<\/[^P])[^\s]*\s)*<P=\"[^~]+~[^~]+~[^~]+~[^\"]+\">\s((([^<]|<[^P]|<P[^=])[^\s]*\s)*<P=\"[^~]+~[^~]+~[^~]+~[^\"]+\">\s(([^<]|<[^P]|<P[^=])[^\s]*\s)*<\/P>\s(([^<]|<[^P]|<P[^=])[^\s]*\s)*)*(([^<]|<[^P\/]|<P[^=]|<\/[^P])[^\s]*\s)*<head[^>]*>\s*([^\s<]+)/) {

	if($ln =~ /\s<P=\"([^~])[^~]*~[^~]+~[^~]+~[^\"]+\">\s((([^<]|<[^P]|<P[^=])[^\s]*\s)*<P=\"[^~]+~[^~]+~[^~]+~[^\"]+\">\s(([^<]|<[^P]|<P[^=])[^\s]*\s)*<\/P>\s(([^<]|<[^P]|<P[^=])[^\s]*\s)*)*(([^<]|<[^P\/]|<P[^=]|<\/[^P])[^\s]*\s)*<P=\"[^~]+~[^~]+~[^~]+~[^\"]+\">\s((([^<]|<[^P]|<P[^=])[^\s]*\s)*<P=\"[^~]+~[^~]+~[^~]+~[^\"]+\">\s(([^<]|<[^P]|<P[^=])[^\s]*\s)*<\/P>\s(([^<]|<[^P]|<P[^=])[^\s]*\s)*)*(([^<]|<[^P\/]|<P[^=]|<\/[^P])[^\s]*\s)*<head[^>]*>\s*([^\s<]+)/) {

		# print "FOUND \n";
		$hdwd = $1;
		$tmpln = $&;
		$target = "";
		if($tmpln =~ /\s([^\s<]+)$/) {
			$target = $1;
		}
		# print "Target: $target\n";
		## if($hdwd eq $target)	{ next; }
	}
	if($hdwd eq "UNDEFINED") { next; }
	$HEADS{$hdwd}++;
}

# CHECK IF count ok and print to file
foreach $hdwd (sort keys %HEADS) {
	if($HEADS{$hdwd} > $MIN) {
		print OP "PHDpos $hdwd\n";
	}
}

# Subroutine to print help message
sub help{
    print "Usage: learnParse4.pl [OPTIONS] OUTPUT SOURCE\n\n"; 
    print "This program learns parent phrases as features from a given training file, SOURCE. Those features which occur at least a user specified number of times (MIN) constitute a learned feature. Each learned feature is printed to the OUTPUT file on a new line. The feature is preceeded with 'PHDpos'.\n\n";
    print "OPTIONS:\n\n";
    print "  --mincount MIN     the feature must occur atleast MIN times to\n";
    print "                     be selected.\n";
    print "  --help             Prints the help message.\n";
    print "  --version          Prints the version number.\n\n"; }

sub usage{
    print "Usage: learnParse4.pl [OPTIONS] OUTPUT SOURCE\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type learnParse4.pl --help for help.\n";}
# function to output the version number
sub showVersion
{
    print STDERR "learnParse4.pl      -        version 0.1\n";
    print STDERR "A component of SyntaLex version 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

