#!/usr/bin/perl -w

#### LAST UPDATED: Jul, 2003 ####
# learnSurf.pl version 0.1

###########################################################################
#
# This program learns surface form features from a given training file, 
# SOURCE.
#
# USAGE :  learnSurf.pl [OPTIONS] OUTPUT SOURCE
#
# OPTIONS:
#     --mincount MIN	 the feature must occur atleast MIN times to 
#			 qualify. Default value is 0.
#     --help             Prints the help message.\n";
#     --version          Prints the version number.\n\n"; 
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("mincount=i", "version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}
# Capture the mincount
if ( defined $opt_mincount )
{
    $MIN = $opt_mincount;
    # CHECK for valid MIN
}
else {
    print "Assuming mincount to be 0\n";
    $MIN = 0;
}

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 1 ) {
    &usage();
    exit;}

$opfl = shift();
$inpfl = shift();

if(-f $opfl) {  
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
	print "Overwriting the file.\n"; }
    else{
	exit; } }
 
# Open the various files
open(INP,"<$inpfl");
open(OP,">$opfl");

# Read lines from input file
while($ln = <INP>) {
	# Ignore blank lines
	if($ln =~ /^\s*$/){
	    next;}

	# Capture pos at the position
	$hdwd = "UNDEFINED";
	if($ln =~ /<head[^>]*>\s*([^\s<]+)/) {
		# print "FOUND \n";
		$hdwd = $1;
	}
	if($hdwd eq "UNDEFINED") { next; }
	$HEADS{$hdwd}++;
}

# CHECK IF count ok and print to file
foreach $hdwd (sort keys %HEADS) {
	if($HEADS{$hdwd} > $MIN) {
		print OP "TGT $hdwd\n";
	}
}

# Subroutine to print help message
sub help{
    print "Usage: learnSurf.pl [OPTIONS] OUTPUT SOURCE\n\n"; 
    print "This program learns surface form features from a given training file, SOURCE.\n\n";
    print "OPTIONS:\n\n";
    print "  --mincount         any surface form feature must must exist more than this\n";
    print "                     minimum number in SOURCE. Defaults to 0.\n";
    print "  --help             Prints the help message.\n";
    print "  --version          Prints the version number.\n\n"; }

sub usage{
    print "Usage: learnSurf.pl [OPTIONS] OUTPUT SOURCE\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type learnSurf.pl --help for help.\n";}
# function to output the version number
sub showVersion
{
    print STDERR "learnSurf.pl      -        version 0.1\n";
    print STDERR "A component of SyntaLex version 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

