#!/usr/bin/perl -w

#### LAST UPDATED: Nov15, 2002 ####
# percent2tilda.pl version 0.1

###########################################################################
#
# This program replaces the percent symbol in the answer ID tag of a 
# Senseval-2 data format training file, with '~'. 
#
# USAGE :  percent2tilda.pl [OPTIONS] SOURCE
# 	   OPTIONS:
#          	--version       Prints the version number.
#          	--help          Prints help message.
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}


# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 0 ) {
    print "Insufficient command line arguments.\n";
    &usage();
    exit;}

$inpfl = shift();
# Check if SOURCE exists
if(!(-f $inpfl)) {
    print "SOURCE file does not exist.\n";
    &usage();
    exit; }

# Open the various files
open(INP,"<$inpfl");

# Read lines from input file
while($ln = <INP>) {
    # Set variable based on whether line part of context or not
    if($ln =~ /^\s*<answer instance=/) {
	$ln =~ s/\%/\~/g;	
    }

    print $ln; }

# Subroutine to print help message
sub help{
    print "Usage: percent2tilda.pl [OPTIONS] SOURCE\n\n"; 
    print "This program replaces the percent symbol in the answer ID tag of a Senseval-2 data format training file with '~'.\n\n";
    print "OPTIONS:\n";
    print "  --help             Prints the help message.\n";
    print "  --version          Prints the version number.\n\n"; }

sub usage{
    print "Usage: percent2tilda.pl [OPTIONS] SOURCE\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type percent2tilda.pl --help for help.\n";}
# function to output the version number
sub showVersion
{
    print STDERR "percent2tilda.pl      -        version 0.1\n";
    print STDERR "A component of SyntaLex version 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

