#!/usr/local/bin/perl -w

#### LAST UPDATED: Oct, 2003 ####
# pos2regex.pl version 0.1

###########################################################################
#
# This program creates regexes corresponding to the features listed in
# the SOURCE file. Each line in the SOURCE file corresponds to one
# feature. It has the position a space and the part of speech.
# The regex created if matched with any line suggests that the line
# has the feature i,e the line has the part of speech at the position.
#
# USAGE :  pos2regex.pl [OPTIONS] OUTPUT SOURCE 
#
# OPTIONS:
#     --help             Prints the help message.
#     --version          Prints the version number.
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 1 ) {
    print "Insufficient command line arguments.\n";
    &usage();
    exit;}

$opfl = shift();
$inpfl = shift();

if(-f $opfl) {  
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
	print "Overwriting the file.\n"; }
    else{
	exit; } }
 
# Open the various files
open(INP,"<$inpfl");
open(OP,">$opfl");

# Read lines from input file
while($ln = <INP>) {
	# Ignore blank lines
	if($ln =~ /^\s*$/){
	    next;}

	if($ln =~ /^\s*[Ll]/) {
		# LEFT Feature
		# Extract N
		$N = "UNDEFINED";
		if($ln =~ /^\s*[lL](\d+)/) { $N = $1; }
		if($N eq "UNDEFINED") {
			print"Invalid value for Left position: $ln\n";
			exit;
		}
		# Capture pos at the position
		$pos = "UNDEFINED";
		if($ln =~ /^\s*[lL]\d+\s+([^\s]+)/) { $pos = $1; }
		if($pos eq "UNDEFINED") {
                        print"Invalid entry for Left position: $ln\n";
                        exit;
                }

		# Escape the $ in PRP$, period, braces etc
		while($pos =~ /(^|[^\\])[\$\(\)\.\?\"]/) {
			$pos =~ s/(^|[^\\])([\$\(\)\.\?\"])/$1\\$2/g;
		}

		$skip = $N - 1;
		# Print Regex
		$reg = "/\\s<p=\\\"$pos\\\"\\/>\\s((([^<]|<[^p]|<p[^=])[^\\s]*\\s)*<p=\\\"[^\\\"]+\\\"\\/>\\s){$skip}(([^<]|<[^p]|<p[^=])[^\\s]*\\s)*<head/";
		print OP "$reg \@name = L$N-$pos\n";
	} elsif($ln =~ /^\s*[rR]/) {
		# RIGHT Feature
		# Extract N
                $N = "UNDEFINED";
                if($ln =~ /^\s*[rR](\d+)/) { $N = $1; }
                if($N eq "UNDEFINED") {
                        print"Invalid value for Right position: $ln\n";
                        exit;
                }
                # Capture pos at the position
                $pos = "UNDEFINED";
		if($ln =~ /^\s*[rR]\d+\s+([^\s]+)/) { $pos = $1; }
                if($pos eq "UNDEFINED") {
                        print"Invalid entry for Right position: $ln\n";
                        exit;
                }

		# Escape the $ in PRP$, period, braces etc
                while($pos =~ /(^|[^\\])[\$\(\)\.\?\"]/) {
                        $pos =~ s/(^|[^\\])([\$\(\)\.\?\"])/$1\\$2/g;
                }

                $skip = $N - 1;
		# Print Regex
                $reg = "/<\\/head>\\s((([^<]|<[^p]|<p[^=])[^\\s]*\\s)*<p=\\\"[^\\\"]+\\\"\\/>\\s){$skip}(([^<]|<[^p]|<p[^=])[^\\s]*\\s)*<p=\\\"$pos\\\"\\/>/";
		print OP "$reg \@name = R$N-$pos\n";
	} 
	else {
		if($ln =~ /^\s*([^\s]+)\s+([^\s]+)/) {
			$Cposition = $1; 
			$Cpos = $2; }


		$#indpositions = -1;
                @indpositions = split(/-/, $Cposition);
                $indpositions[0] =~ s/^..//;


		$#indpos = -1;
                @indpos = split(/-/, $Cpos);
	
		undef %RpositionH;	
		undef %LpositionH;
		$posctr=0;
		foreach $position (@indpositions) {
			if($position =~ /^[Ll]/) {
				$position =~ s/^.//;
				$position = $position + 0;
				$LpositionH{$position} = $indpos[$posctr];
			} else {
				$position =~ s/^.//;
				$RpositionH{$position} = $indpos[$posctr];
			}
			$posctr++;
		}

		$prevposition = 0;
		$Lreg = "<head";
		$Rreg = "<\\\/head>\\s+";
		sub numerically { $a <=> $b }
        	foreach $position (sort numerically keys %LpositionH) {
			$pos = "UNDEFINED";
			$pos = $LpositionH{$position};
			if($pos eq "UNDEFINED")	{ print "Undefined pos\n"; exit; }

                	# Escape the $ in PRP$, period, braces etc
                	while($pos =~ /(^|[^\\])[\$\(\)\.\?\"]/) {
                        	$pos =~ s/(^|[^\\])([\$\(\)\.\?\"])/$1\\$2/g;
                	}

                		$skip = $position - $prevposition - 1;
                		# Left Regex
                		$Lreg = "<p=\\\"$pos\\\"\\/>\\s((([^<]|<[^p]|<p[^=])[^\\s]*\\s)*<p=\\\"[^\\\"]+\\\"\\/>\\s){$skip}(([^<]|<[^p]|<p[^=])[^\\s]*\\s)*".$Lreg;
				$posctr++;
				$prevposition = $position;
			}

		$prevposition = 0;
		foreach $position (sort numerically keys %RpositionH) {
                        $pos = "UNDEFINED";
                        $pos = $RpositionH{$position};
                        if($pos eq "UNDEFINED") { print "Undefined pos\n"; exit; }

                        # Escape the $ in PRP$, period, braces etc
                        while($pos =~ /(^|[^\\])[\$\(\)\.\?\"]/) {
                                $pos =~ s/(^|[^\\])([\$\(\)\.\?\"])/$1\\$2/g;
                        }

                                $skip = $position - $prevposition - 1;
                                # Left Regex
                                $Rreg = $Rreg."((([^<]|<[^p]|<p[^=])[^\\s]*\\s)*<p=\\\"[^\\\"]+\\\"\\/>\\s){$skip}(([^<]|<[^p]|<p[^=])[^\\s]*\\s)*<p=\\\"$pos\\\"\\/>\\s+";
                                $posctr++;
                                $prevposition = $position;
                        }

		$reg = "/".$Lreg."[^<]+".$Rreg."/";
                print OP "$reg \@name = $Cposition-$Cpos\n";
	}
}

# Subroutine to print help message
sub help{
    print "Usage: pos2regex.pl [OPTIONS] OUTPUT SOURCE\n\n"; 
    print "This program creates regexes corresponding to the features listed in the SOURCE file. Each line in the SOURCE file corresponds to one feature. It has the position a space and the part of speech. The regex created if matched with any line suggests that the line has the feature i,e the line has the part of speech at the position.\n\n";
    print "OPTIONS:\n\n";
    print "  --help             Prints the help message.\n";
    print "  --version          Prints the version number.\n\n"; }

sub usage{
    print "Usage: pos2regex.pl [OPTIONS] OUTPUT SOURCE\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type pos2regex.pl --help for help.\n";}
# function to output the version number
sub showVersion
{
    print STDERR "pos2regex.pl      -        version 0.1\n";
    print STDERR "A component of SyntaLex version 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

