#!/usr/bin/perl -w

#### LAST UPDATED: Feb, 2004 ####
# red.pl version 0.1

###########################################################################
#
# This program takes as input a file which gives the intended sense for 
# given test instances. It also takes the files which were produced by
# two systems intended to do word sense disambiguation. It then gives as
# output: 
#
# Total number of instances
# Number of instances attempted - system 1 
# Number of instances attempted - system 2 
# Number of instances correct - system1 
# Number of instances correct - system2
# Number of instances correctly tagged by either system1 or system2
# Optimal Ensemble
# Number of instances tagged the same by systems 1 and 2 (similarity in tagging by dice)
# Number of instances that are assigned atleast one sense in common by Systems 1 and 2
# A subset of these which are correct 
# Precision on Agreement 
# Conservative ensemble 
#
# USAGE : red.pl [OPTIONS] GOLD SYSTEM1 SYSTEM2 
# OPTIONS:
#       --version       Prints the version number.
#       --help          Prints help message.
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version","help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help )
{
    &help();
    exit;
}
# if version has been requested, show version
if ( defined $opt_version ) {
    $opt_version = 1;
    &showVersion();
    exit; }

# Check if commandline options have been provided
# If not print usage information
if ( $#ARGV < 2 )
{
    print "Insufficient command line arguments.\n";
    &usage();
    exit;
}

# Accept DESTINATION file
$goldfl = shift(@ARGV);
$sys1fl = shift(@ARGV);
$sys2fl = shift(@ARGV);

# Open files 
open(GLD, "<$goldfl") or die "could not open file ";
open(SYS1, "<$sys1fl") or die "could not open file ";
open(SYS2, "<$sys2fl") or die "could not open file ";

# Define hashes
my %sys1count; my %sys2count; my %gldcount;
$numinstances = 0;

# Initialise and start reading from GOLD file - line by line
while(<GLD>)
{
	# Ignore blank lines
	if (/^\s*$/)	{ next; }

	if(/^\s*[^\s]+\s+([^\s]+)\s+([^\s].*)$/)
	{
		# Capture instance ID ans Answers
		$inst = $1;
		$ansset = $2;
		# Ignore P tags
		$ansset =~ s/(^|\s)P(\s|$)/ /g;
		# print "Pans: $ansset\n";

		# Check if atleast one sense given
		if($ansset =~ /^\s*$/) {
			print "ERROR: No sense assigned for instance ID $inst\n"; exit; }

		# Eliminate trailing and pre-ceeding spaces
                $ansset =~ s/\s*$//;
		$ansset =~ s/^\s*//;	

		# Capture individual answers
		$#indans = -1;
		@indans = split(/\s+/, $ansset);

		if($gld{$inst})
		{
			print "WARNING GOLD FILE HAS MULTIPLE ENTRIES FOR SAME INSTANCE: $inst\n\n";
		}
		# Store in hash
		# $gld{$inst} = $anss;
		foreach $anss (@indans) {
			$gld{$inst}{$anss}=1;
			$gldcount{$inst}++;
			# print "gld anss: $anss\n";
		}
		

		# count number of instances
		$numinstances++;
	}
	else {
		print "ERROR: Entry not in format or with no answer ID:\n";
		print;
	}
}

$sys1rt = 0;
$sys2rt = 0;

# Read lines from System1 and store correct data in hash
while(<SYS1>)
{
	# Ignore blank lines
        if (/^\s*$/)    { next; }

	if(/^\s*[^\s]+\s+([^\s]+)\s+([^\s].*)$/)
        {
                # Capture instance ID ans Answers
                $inst = $1;
                $ansset = $2;
		$sys1at++;

		# Ignore P tags
                $ansset =~ s/(^|\s)P(\s|$)/ /g;
                # Check if atleast one sense given
                if($ansset =~ /^\s*$/) {
                        print "ERROR: No sense assigned for instance ID $inst\n"; exit; }

		# Eliminate trailing and pre-ceeding spaces
                $ansset =~ s/\s*$//;
                $ansset =~ s/^\s*//;

		# print "ansset: $ansset\n";
                # Capture individual answers
		$#indans = -1;
                @indans = split(/\s+/, $ansset);

		#
		$tmprt = 0;
		$tmpwr = 0;
		foreach $anss (@indans) {
			# print "anss: $anss\n";
			$allsys1{$inst}{$anss} = 1;
			$sys1count{$inst}++;

			if(exists $gld{$inst}{$anss})
			{
				$tmprt++;
				# $sys1{$inst}{$anss} = 1;
			}	
			else {
				# Atleast one sense mismatch
				$tmpwr++;
			}
		}
		# Calculate the partial score
                # $Pscore = 2 * $tmprt / ($sys1count{$inst} + $gldcount{$inst});
		$Pscore = $tmprt / $sys1count{$inst};
		$sys1Pscore{$inst} = $Pscore;
		$sys1rt += $Pscore;
	}
}

# Read lines from System2 and store correct data in hash
while(<SYS2>)
{
	# Ignore blank lines
        if (/^\s*$/)    { next; }

        if(/^\s*[^\s]+\s+([^\s]+)\s+([^\s].*)$/)
        {
                # Capture instance ID ans Answers
                $inst = $1;
                $ansset = $2;
		$sys2at++;

		# Ignore P tags
                $ansset =~ s/(^|\s)P(\s|$)/ /g;
                # Check if atleast one sense given
                if($ansset =~ /^\s*$/) {
                        print "ERROR: No sense assigned for instance ID $inst\n"; exit; }

                # Eliminate trailing and pre-ceeding spaces
                $ansset =~ s/\s*$//;
                $ansset =~ s/^\s*//;

                # Capture individual answers
		$#indans = -1;
		@indans = split(/\s+/, $ansset);

		$tmprt = 0;
                $tmpwr = 0;
                foreach $anss (@indans) {
			$allsys2{$inst}{$anss} = 1;
                        $sys2count{$inst}++;
			# print "sys2 anss: $anss\n";

                        if(exists $gld{$inst}{$anss})
                        {
                                $tmprt++;
                                # $sys2{$inst}{$anss} = 1;
                        }
                        else {
                                # Atleast one sense mismatch
                                $tmpwr++;
                        }
                }
		# Calculate the partial score
                # $Pscore = 2 * $tmprt / ($sys2count{$inst} + $gldcount{$inst});
		$Pscore = $tmprt / $sys2count{$inst};
		$sys2Pscore{$inst} = $Pscore;
                $sys2rt += $Pscore;
        }
}

$agreeinstances = 0;
# Check Redundancy in systems, irrespective of correctness
foreach $inst1 (keys %allsys1)
{
	$tmprt = 0;
        $tmpwr = 0;
	$tmpok = 0;
        foreach $ans1 (keys %{$allsys1{$inst1}}) {
                if(exists $allsys2{$inst1}{$ans1}) { 
			$tmprt++; 
			if(exists $gld{$inst1}{$ans1}) {
				$tmpok++;
			}
		}
                else { $tmpwr++; }
        }       
	# Dice
         $Pscore = 2 * $tmprt / ($sys1count{$inst1} + $sys2count{$inst1});
         $allred += $Pscore;
	# $Cred += ($Pscore * ($sys1Pscore{$inst1} + $sys2Pscore{$inst1}) / 2);
	# $Cred += $tmprt / 
	# Cumulative precision on agreement
	if($tmprt == 0) { $pagree += 0; }
	else { $pagree += ($tmpok/$tmprt); 
		$agreeinstances++; }
	
	# Choosing the best assignment
	$optimal = $sys1Pscore{$inst1};
	if($sys1Pscore{$inst1} < $sys2Pscore{$inst1}) {
		$optimal = $sys2Pscore{$inst1}; }
	$cumOptimal += $optimal;	
}

if ($numinstances == 0) {
	print "Number of instances is Zero. Exiting.\n";
	exit;
}

# Agreement
$agreement = $agreeinstances * 100 / $numinstances;

# precion on agreement
if ($agreeinstances > 0) {
	$Ppagree = $pagree * 100 / $agreeinstances;
}
else {
	$Ppagree = "UNDEFINED";
}
$PcumOptimal = $cumOptimal * 100 / $numinstances;

# Calcullate percentages
$Psys1at = $sys1at * 100 / $numinstances;
$Psys2at = $sys2at * 100 / $numinstances;
$Psys1rt = $sys1rt * 100 / $numinstances;
$Psys2rt = $sys2rt * 100 / $numinstances;
$Pallred = $allred * 100 / $numinstances;

# $conensemble = $Ppagree * $agreement / 100;
$conensemble = $pagree * 100 / $numinstances;

# Printing output
print "Total number of instances: $numinstances 
Number of instances attempted - system 1: $sys1at [";
printf ("%3.2f", $Psys1at);
print "\%]
Number of instances attempted - system 2: $sys2at [";
printf ("%3.2f", $Psys2at);
print "\%]
Number of instances correct - system1: ";
printf ("%3.2f [%3.2f", $sys1rt, $Psys1rt);
print "\%] 
Number of instances correct - system2: ";
printf ("%3.2f [%3.2f", $sys2rt, $Psys2rt);
print "\%] 
Number of instances correctly tagged by either system1 or system2: ";
printf ("%3.2f", $cumOptimal);
print "\nOptimal Ensemble: ";
printf ("%3.2f", $PcumOptimal);
print "\%
Number of instances tagged the same by systems 1 and 2 (similarity in tagging by dice): ";
printf ("%3.2f [%3.2f", $allred, $Pallred);
print "]\nNumber of instances that are assigned atleast one sense in common by Systems 1 and 2: ";
printf ("%3.2f [%3.2f", $agreeinstances, $agreement);
print "\%]\nA subset of these which are correct: ";
printf ("%3.2f", $pagree);
print "\nPrecision on Agreement: ";
if ($agreeinstances > 0) {
	printf ("%3.2f", $Ppagree);
	print "\%\n";
} else {
	print "UNDEFINED\n";
}
print "Conservative ensemble: ";
printf ("%3.2f", $conensemble);
print "\%\n";

# Subroutine to print help message
sub help
{
    print "Usage: red.pl [OPTIONS] GOLD SYSTEM1 SYSTEM2\n\n";
    print "This program takes as input a file which gives the intended sense for given test instances. It also takes the files which were produced by two systems intended to do word sense disambiguation. It then gives as output various measures which shed light on the redundancy and complementarity of the answer files.\n";
    print "OPTIONS:\n";
    print "  --version    Prints the version number.\n";
    print "  --help       Prints the help message.\n";
}

sub usage
{
    print "Usage: red.pl [OPTIONS] GOLD SYSTEM1 SYSTEM2\n";
    AskHelp();
}
# Message describing a way to get help
sub AskHelp
{
    print "Type red.pl --help for help.\n";
}
# function to output the version number
sub showVersion
{
    print STDERR "red.pl      -        version 0.1\n";
    print STDERR "A component of SyntaLex 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad and Ted Pedersen\n";
}


