#!/usr/bin/perl -w

#### LAST UPDATED: Sep, 2003 ####
# removePtag.pl version 0.1

###########################################################################
#
# This program takes a Senseval-2 format training file as input (SOURCE).
# It eliminates those xml tags which suggest that the instance is a 
# proper noun. For example:
#
#	<answer instance="art.40244" senseid="P"/>
#
# P stands for proper noun and is not another sense of the target word.
#
# USAGE :  removePtag.pl [OPTIONS] DESTINATION SOURCE
# OPTIONS:
#       --version       Prints the version number.
#       --help          Prints help message.
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version ) {
    $opt_version = 1;
    &showVersion();
    exit; }

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 1 ) {
    print "Insufficient command line arguments.\n";
    &usage();
    exit;}

# Accept SOURCE and DESTINATION names
$opfl = shift();
$inpfl = shift();

# If output file exists, ask permission to over-write
if(-e $opfl) {  
    print "File/Directory with same name as DESTINATION already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
	print "Overwriting the file.\n"; }
    else{
	exit; } }

# Check if SOURCE exists
if(!(-f $inpfl)) {
    print "SOURCE file does not exist.\n";
    &usage();
    exit; } 

# Open the various files
open(INP,"<$inpfl");
open(OP,">$opfl");

$ctr = 1;
# Read lines from input file
while($ln = <INP>) {
	# Reset sense tag counter
	if($ln =~ /^\s*<instance id=\"([^\"]+)\"/) { 
		$instID = $1;
		$ctr = 0; }
	# Eliminate P tags
	if($ln =~ /^\s*<answer instance=\"[^\"]+\" senseid=\"P\"\/>\s*$/){
		next; }
	# Increment counter
	if($ln =~ /^\s*<answer instance=\"/) 	{ $ctr++; }

	if($ln =~ /^\s*<context>\s*$/) {
		if ($ctr == 0)  { 
                        # print "$instID\n"; 
			print OP "<answer instance=\"$instID\" senseid=\"U\"\/>\n";	
                }
	}
    	print OP $ln; 
}

# Subroutine to print help message
sub help{
    print "Usage: removePtag.pl [OPTIONS] DESTINATION SOURCE\n\n"; 
    print "This program takes a Senseval-2 format training file as input (SOURCE). It eliminates those xml tags which suggest that the instance is a proper noun. For example in : <answer instance=\"art.40244\" senseid=\"P\"/> : P stands for proper noun and is not another sense of the target word.\n\n";
    print "OPTIONS:\n\n";
    print "  --version    Prints the version number.\n";
    print "  --help       Prints the help message.\n\n";}

sub usage{
    print "Usage: removePtag.pl [OPTIONS] DESTINATION SOURCE\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type removePtag.pl --help for help.\n";}
# function to output the version number
sub showVersion
{
    print STDERR "removePtag.pl      -        version 0.1\n";
    print STDERR "A component of SyntaLex 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad and Ted Pedersen\n";
}

