#!/usr/bin/perl -w

#### LAST UPDATED: Feb, 2004 ####
# splitpos.pl version 0.1

###########################################################################
#
# This program prints out precision value averaged for each of the tasks
# specified as command line arguments. It picks up the reasults for each
# individual lexelt from the .scores file SCORES.
# Note: By task I mean instance ID of the test instance.
#
# USAGE :  splitpos.pl [OPTIONS] SCORES TASK1 [TASK2 ...] > OUTPUT
#
#       --version       Prints the version number.
#       --help          Prints help message.
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version ) {
    $opt_version = 1;
    &showVersion();
    exit; }

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 1 ) {
    print "Insufficient command line arguments.\n";
    &usage();
    exit;}

# Accept SOURCE and DESTINATION names
$inpfl = shift();

# Check if SOURCE exists
if(!(-f $inpfl)) {
    print "SOURCE file does not exist.\n";
    &usage();
    exit; } 

# Open the various files
open(INP,"<$inpfl");

# Initialize scores for each lexlet
while($lex = shift()) {
	$lex =~ s/\.[a-z]$//;
	$Hscore{$lex} = 1;
}

$cumscore = 0;
$total = 0;

# Read lines from input file
while($ln = <INP>) {
	# CHANGE: Jan26
	# if($ln =~ /^\s*score for \"([^\_]+)\_[^:]+:\s*([^\s]+)\s*$/){
	if($ln =~ /^\s*score for \"([^\_]+)\_[^\"]+\":\s*([^\s]+)\s*$/){
	    $task = $1;
	    $score = $2;
	    # DBUG
	    # print "task = $task    score = $score\n";
	    if(exists $Hscore{$task}) {
		$cumscore += $score;
		$total += 1;
	    }
	    next;}
}

if ($total == 0) { exit; }

$prec = $cumscore * 100 / $total;
printf "Accuracy: $cumscore/$total = %3.2f", $prec;
print "\%\n";

# Subroutine to print help message
sub help{
    print "Usage: splitpos.pl [OPTIONS] SCORES TASK1 [TASK2 ...] > OUTPUT\n\n"; 
    print "This program prints out precision value averaged for each of the tasks specified as command line arguments. It picks up the reasults for each individual lexelt from the .scores file SCORES.\n\n"; 
    print "OPTIONS:\n";
    print "  --version    Prints the version number.\n";
    print "  --help       Prints the help message.\n\n";}

sub usage{
    print "Usage: splitpos.pl [OPTIONS] SCORES TASK1 [TASK2 ...] > OUTPUT\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type splitpos.pl --help for help.\n";}
# function to output the version number
sub showVersion
{
    print STDERR "splitpos.pl      -        version 0.1\n";
    print STDERR "A component of SyntaLex 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad and Ted Pedersen\n";
}

