#!/usr/bin/perl -w

#### LAST UPDATED: Oct, 2003 ####
# surf2regex.pl version 0.1

###########################################################################
#
# This program creates regexes corresponding to the surface form features 
# listed in the SOURCE file. Each line in the SOURCE file corresponds to one
# feature. Each entry consists of 'TGT', space and a surface form (type).
# TGT stands for the surface form of the target word.
# The regex created if matched with any line suggests that the line
# has the feature i,e the line has target word in the particular surface 
# form.
#
# USAGE :  surf2regex.pl [OPTIONS] OUTPUT SOURCE 
#
# OPTIONS:
#     --help             Prints the help message.
#     --version          Prints the version number.
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 1 ) {
    print "Insufficient command line arguments.\n";
    &usage();
    exit;}

$opfl = shift();
$inpfl = shift();

if(-f $opfl) {  
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
	print "Overwriting the file.\n"; }
    else{
	exit; } }
 
# Open the various files
open(INP,"<$inpfl");
open(OP,">$opfl");

# Read lines from input file
while($ln = <INP>) {
	# Ignore blank lines
	if($ln =~ /^\s*$/){
	    next;}

	if($ln =~ /^\s*TGT\s/) {
		# Feature: Surface form
		# Capture target word
		$hdwd = "UNDEFINED";
		if($ln =~ /^\s*TGT\s+([^\s]+)/) { $hdwd = $1; }
		if($hdwd eq "UNDEFINED") {
                        print"Invalid entry for head word: $ln\n";
                        exit;
                }

		# Escape the $ in PRP$, period, braces etc
                while($hdwd =~ /(^|[^\\])[\$\(\)\.\?\"]/) {
                        $hdwd =~ s/(^|[^\\])([\$\(\)\.\?\"])/$1\\$2/g;
                }
		
		# Print Regex
		# HDwd
		# $reg = "/\\s<P=\\\"[^~]+~$hdwd~[^~]+~[^\\\"]+\\\">\\s((([^<]|<[^P]|<P[^=])[^\\s]*\\s)*<P=\\\"[^~]+~[^~]+~[^~]+~[^\\\"]+\\\">\\s(([^<]|<[^P]|<P[^=])[^\\s]*\\s)*<\\/P>\\s(([^<]|<[^P]|<P[^=])[^\\s]*\\s)*)*<head/";

		$reg = "/<head[^>]*>\\s*$hdwd\\s*</";
		print OP "$reg \@name = TGT-$hdwd\n";
	} 
}

# Subroutine to print help message
sub help{
    print "Usage: surf2regex.pl [OPTIONS] OUTPUT SOURCE\n\n"; 
    print "This program creates regexes corresponding to the surface form features listed in the SOURCE file. Each line in the SOURCE file corresponds to one feature. Each entry consists of 'TGT', space and a surface form (type). TGT stands for the surface form of the target word. The regex created if matched with any line suggests that the line has the feature i,e the line has target word in the particular surface form.\n\n";
    print "OPTIONS:\n\n";
    print "  --help             Prints the help message.\n";
    print "  --version          Prints the version number.\n\n"; }

sub usage{
    print "Usage: surf2regex.pl [OPTIONS] OUTPUT SOURCE\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type surf2regex.pl --help for help.\n";}
# function to output the version number
sub showVersion
{
    print STDERR "surf2regex.pl      -        version 0.1\n";
    print STDERR "A component of SyntaLex version 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

